@@ -21,25 +21,46 @@ advector <- function(x) {
     }
     ans
 }
-## Make this object AD interpretable if an AD context is active
-## NOTE: Should be needed rarely!
-## - Intended for objects that prevent simple S3 method dispatch
-## - By defatult does *nothing* if *not* in an active AD context
-## - To see what the 'magic' object would look like pass condition=TRUE
-## Example 1: Starting out with a sparse matrix requires a little magic
-## D <- Matrix::.symDiagonal(10)
-## magic(D, TRUE) ## advector with attributes
-magic <- function(x, condition = ad_context()) {
-    if (!condition) return (x)
-    if (is(x, "advector")) return (x)
-    if (is(x, "sparseMatrix")) {
+##' Convert R object to AD
+##'
+##' Signify that this object should be given an AD interpretation if evaluated in an active AD context. Otherwise, keep object as is.
+##' @details \code{AD} is a generic constructor, converting plain R structures to RTMB objects if in an autodiff context. Otherwise, it does nothing (and adds virtually no computational overhead).
+##'
+##' \code{AD} knows the following R objects:
+##'
+##' - Numeric objects from \pkg{base}, such as `numeric()`, `matrix()`, `array()`, are converted to class \link{advector} with other attributes kept intact.
+##' - Complex objects from \pkg{base}, such as `complex()`, are converted to class \link{adcomplex}.
+##' - Sparse matrices from \pkg{Matrix}, such as `Matrix()`, `Diagonal()`, are converted to `adsparse` which is essentially a `dgCMatrix` with \link{advector} x-slot.
+##'
+##' \code{AD} provides a reliable way to avoid problems with method dispatch when mixing operand types. For instance, sub assigning `x[i] <- y` may be problematic when `x` is numeric and `y` is `advector`. A prior statement `x <- AD(x)` solves potential method dispatch issues and can therefore be used as a reliable alternative to \link{ADoverload}.
+##' @param x Object to be converted.
+##' @param force Logical; Force AD conversion even if no AD context? (for debugging)
+##' @examples
+##' ## numeric object to AD
+##' AD(numeric(4), force=TRUE)
+##' ## complex object to AD
+##' AD(complex(4), force=TRUE)
+##' ## Convert sparse matrices (Matrix package) to AD representation
+##' F <- MakeTape(function(x) {
+##'   M <- AD(Matrix::Matrix(0,4,4))
+##'   M[1,] <- x
+##'   D <- AD(Matrix::Diagonal(4))
+##'   D@x[] <- x
+##'   M + D
+##' }, 0)
+##' F(2)
+AD <- function(x, force = FALSE) {
+    if (!force && !ad_context()) return (x)
+    if (inherits(x, "advector")) return (x)
+    if (inherits(x, "anysparse")) {
         x <- as(x, "adsparse")
-        return (x)
-    } else if (is.numeric(x)) {
+    } else if (is.double(x) || is.integer(x) || is.logical(x)) {
         x <- advector(x)
-        return (x)
+    } else if (is.complex(x)) {
+        x <- adcomplex(advector(Re(x)), advector(Im(x)))
     } else
-        stop("'magic' does not know this object")
+        stop("'AD' does not know this object")
+    x
 }
 .Compare <- getGroupMembers("Compare")
 ##' @describeIn ADvector Binary operations
@@ -60,7 +81,7 @@ magic <- function(x, condition = ad_context()) {
 }
 ##' @describeIn ADvector Unary operations
 "Math.advector" <- function(x, ...) {
-    Math1(x, .Generic)
+    Math1(x, .Generic, ...)
 }
 
 ##' @describeIn ADvector Makes \code{array(x)} work.
@@ -178,10 +199,32 @@ detachADoverloads <- function(enable=TRUE, ...) {
 rep.advector <- function (x, ...) {
     as_advector(NextMethod())
 }
+##' @describeIn ADvector Equivalent of \link[base]{is.nan}. Check NaN status of a *constant* `advector` expression. If not constant throw an error.
+is.nan.advector <- function(x) {
+    if (!compare_allow() && any(getVariables(x)))
+        stop("Can only determine NaN status of constant expressions")
+    is.nan(getValues(x))
+}
+##' @describeIn ADvector Equivalent of \link[base]{is.finite}. Check finite status of a *constant* `advector` expression. If not constant throw an error.
+is.finite.advector <- function(x) {
+    if (!compare_allow() && any(getVariables(x)))
+        stop("Can only determine finite status of constant expressions")
+    is.finite(getValues(x))
+}
+##' @describeIn ADvector Equivalent of \link[base]{is.infinite}. Check infinity status of a *constant* `advector` expression. If not constant throw an error.
+is.infinite.advector <- function(x) {
+    if (!compare_allow() && any(getVariables(x)))
+        stop("Can only determine infinity status of constant expressions")
+    is.infinite(getValues(x))
+}
+##' @describeIn ADvector Equivalent of \link[base]{is.na}. Check NA status of an `advector`. NAs can only occur directly (as constants) or indirectly as the result of an operation with NA operands. For a tape built with non-NA parameters the NA status of any expression is constant and can therefore safely be used as part of the calculations. (assuming correct propagation of NAs via C-level arithmetic).
+is.na.advector <- function(x) {
+    is.na(getValues(x))
+}
 ##' @describeIn ADvector Equivalent of \link[base]{sum}. \code{na.rm=TRUE} is allowed, but note that this feature assumes correct propagation of NAs via C-level arithmetic.
 sum.advector <- function(x, ..., na.rm = FALSE) {
     if (na.rm) {
-        x <- x[!is.na(getValues(x))]
+        x <- x[!is.na(x)]
     }
     Reduce1(x, "+") + sum(..., na.rm = na.rm)
 }
@@ -204,10 +247,11 @@ as.double.advector <- function(x, ...) {
     attributes(x) <- attributes(x)["class"]
     x
 }
-##' @describeIn ADvector \link{Complex} operations are not allowed and will throw an error.
+##' @describeIn ADvector \link{Complex} operations are redirected to \link{adcomplex}.
 ##' @param z Complex (not allowed)
-Complex.advector <- function(z)
-    stop("'advector' does not allow complex operations")
+Complex.advector <- function(z) {
+    callGeneric(adcomplex(z))
+}
 ##' @describeIn ADvector Non differentiable \link{Summary} operations (e.g. \code{min} \code{max}) are not allowed and will throw an error.
 Summary.advector <- function(..., na.rm = FALSE)
     stop("'advector' does not allow operation ", sQuote(.Generic))
@@ -525,6 +569,8 @@ observation.name <- NULL
 data.term.indicator <- NULL
 data <- NULL
 ##' @describeIn TMB-interface Interface to \link[TMB]{MakeADFun}.
+##' @details \link{MakeADFun} builds a TMB model object mostly compatible with the \pkg{TMB} package and with an almost identical interface.
+##' The main difference in \pkg{RTMB} is that the objective function **and** the data is now given via a single argument \code{func}. Because \code{func} can be a *closure*, there is no need for an explicit data argument to \link{MakeADFun} (see examples).
 ##' @param func Function taking a parameter list (or parameter vector) as input.
 ##' @param parameters Parameter list (or parameter vector) used by \code{func}.
 ##' @param random As \link[TMB]{MakeADFun}.
@@ -572,6 +618,7 @@ MakeADFun <- function(func, parameters, random=NULL, profile=NULL, integrate=NUL
                     integrate=NULL,
                     intern=FALSE,
                     ...)
+    TMBArgs[duplicated(names(TMBArgs))] <- NULL
     TMBArgs$DLL <- "RTMB" ## Override if included in ...
     obj <- do.call(TMB::MakeADFun, TMBArgs)
     ## Handling maps (copied and modified parList)
@@ -738,14 +785,38 @@ MakeADFun <- function(func, parameters, random=NULL, profile=NULL, integrate=NUL
     obj
 }
 
+## 'Patch' a TMB function by overloading 'TMB::MakeADFun' with
+## 'RTMB::MakeADFun', possibly accounting for TMB/RTMB argument
+## differences.
+## FIXME (?): The current patching takes place in RTMB namespace at
+## RTMB install time. It follows, that changes to TMB
+## (e.g. TMB::sdreport, TMB::oneStepPredict, ...) require
+## re-installation of RTMB to take place ! We could alternatively
+## apply the patches from inside the '.onLoad' function.
+TMB_patch <- function(fun, ...) {
+    environment(fun) <- new.env( parent = environment(fun) )
+    environment(fun)$MakeADFun <- MakeADFun ## RTMB::MakeADFun
+    dotargs <- list(...)
+    formals(environment(fun)$MakeADFun)[names(dotargs)] <- dotargs
+    fun
+}
+
+## Text substitution in function body (without changing environment)
+bodysub <- function(fun, pattern, replacement, ...) {
+    body(fun) <- parse(text=gsub(pattern,
+                                 replacement,
+                                 deparse(body(fun)),
+                                 ...))[[1]]
+    fun
+}
+sdreport_patch <- TMB_patch(TMB::sdreport)
+sdreport_patch <- bodysub(sdreport_patch,
+                          "\\(.Call\\(.*have_tmb_symbolic.*\\)\\)",
+                          "(FALSE)")
+
 ##' @describeIn TMB-interface Interface to \link[TMB]{sdreport}.
 ##' @param obj TMB model object (output from \link{MakeADFun})
 sdreport <- function(obj, ...) {
-    sdreport_patch <- TMB::sdreport
-    tmb_envir <- environment(sdreport_patch)
-    env <- local({ MakeADFun <- RTMB::MakeADFun; environment() })
-    parent.env(env) <- tmb_envir
-    environment(sdreport_patch) <- env
     sdreport_patch(obj, ...)
 }
 
